@@ -1,8 +1,8 @@
 #' @name CalSF
 #' @aliases CalSF
-#' @title Single frame calibration estimator
+#' @title SF calibration estimator
 #' 
-#' @description Produces estimates for population totals and means using the single frame calibration estimator from survey data obtained
+#' @description Produces estimates for population totals and means using the SF calibration estimator from survey data obtained
 #'  from a dual frame sampling design. Confidence intervals are also computed, if required. 
 #' 
 #' @usage CalSF(ysA, ysB, pi_A, pi_B, pik_ab_B, pik_ba_A, domains_A, domains_B, N_A = NULL,
@@ -32,7 +32,7 @@
 #' @param X (Optional) A numeric value or vector of length \eqn{m_T}, with \eqn{m_T} the number of auxiliary variables in both frames, indicating the population totals for the auxiliary variables considered in both frames.
 #' @param met (Optional) A character vector indicating the distance that must be used in calibration process. Possible values are "linear", "raking" and "logit". Default is "linear".
 #' @param conf_level (Optional) A numeric value indicating the confidence level for the confidence intervals, if desired.
-#' @details Single frame calibration estimator of population total is given by
+#' @details SF calibration estimator of population total is given by
 #'  \deqn{\hat{Y}_{CalSF} = \hat{Y}_a + \hat{Y}_{ab} + \hat{Y}_b}
 #'  where \eqn{\hat{Y}_a = \sum_{i \in s_a}\tilde{d}_i y_i, \hat{Y}_{ab} = \sum_{i \in (s_{ab} \cup s_{ba})}\tilde{d}_i y_i} 
 #'  and \eqn{\hat{Y}_b = \sum_{i \in s_b} \tilde{d}_i y_i}, with \eqn{\tilde{d}_i} calibration weights which are calculated
@@ -59,7 +59,14 @@
 #'  To obtain an estimator of the variance for this estimator, one can use Deville's expression
 #'  \deqn{\hat{V}(\hat{Y}_{CalSF}) = \frac{1}{1-\sum_{k\in s} a_k^2}\sum_{k\in s}(1-\pi_k)\left(\frac{e_k}{\pi_k} - \sum_{l\in s} a_{l} \frac{e_l}{\pi_l}\right)^2}
 #'  where \eqn{a_k=(1-\pi_k)/\sum_{l\in s} (1-\pi_l)} and \eqn{e_k} are the residuals of the regression with auxiliary variables as regressors.
-#' @return A numeric matrix containing estimations of population total and population mean for considered values.
+#' @return \code{CalSF} returns an object of class "EstimatorDF" which is a list with, at least, the following components:
+#'  \item{Call}{the matched call.}
+#'  \item{Est}{total and mean estimation for main variable(s).}
+#'  \item{VarEst}{variance estimation for main variable(s).}
+#'  If parameter \code{conf_level} is different from \code{NULL}, object includes component
+#'  \item{ConfInt}{total and mean estimation and confidence intervals for main variables(s).}
+#'  In addition, components \code{TotDomEst} and \code{MeanDomEst} are available when estimator is based on estimators of the domains. Component \code{Param} shows value of parameters involded in calculation of the estimator (if any).
+#'  By default, only \code{Est} component (or \code{ConfInt} component, if parameter \code{conf_level} is different from \code{NULL}) is shown. It is possible to access to all the components of the objects by using function \code{summary}.
 #' @references Ranalli, M. G., Arcos, A., Rueda, M. and Teodoro, A. (2013)
 #'  \emph{Calibration estimation in dual frame surveys}. arXiv:1312.0761 [stat.ME]
 #' @references Deville, J. C., Sarndal, C. E. (1992)
@@ -67,31 +74,27 @@
 #'  Journal of the American Statistical Association, 87, 376 - 382
 #' @seealso \code{\link{JackCalSF}}
 #' @examples
-#' data(HouseholdsA)
-#' dataA <- attach(HouseholdsA)
-#' detach(HouseholdsA)
-#' data(HouseholdsB)
-#' dataB <- attach(HouseholdsB)
-#' detach(HouseholdsB)
+#' data(DatA)
+#' data(DatB)
 #' data(PiklA)
 #' data(PiklB)
 #' 
-#' #Let calculate single frame calibration estimator for variable Clothing, without
+#' #Let calculate SF calibration estimator for variable Clothing, without
 #' #considering any auxiliary information
-#' CalSF(dataA$Clothing, dataB$Clothing, PiklA, PiklB, dataA$ProbB, dataB$ProbA, 
-#' dataA$Domain, dataB$Domain)
+#' CalSF(DatA$Clo, DatB$Clo, PiklA, PiklB, DatA$ProbB, DatB$ProbA, 
+#' DatA$Domain, DatB$Domain)
 #' 
-#' #Now, let calculate single frame calibration estimator for variable Leisure when the frame
+#' #Now, let calculate SF calibration estimator for variable Leisure when the frame
 #' #sizes and the overlap domain size are known
-#' CalSF(dataA$Leisure, dataB$Leisure, PiklA, PiklB, dataA$ProbB, dataB$ProbA, dataA$Domain, 
-#' dataB$Domain, N_A = 1735, N_B = 1191, N_ab = 601)
+#' CalSF(DatA$Lei, DatB$Lei, PiklA, PiklB, DatA$ProbB, DatB$ProbA, DatA$Domain, 
+#' DatB$Domain, N_A = 1735, N_B = 1191, N_ab = 601)
 #' 
-#' #Finally, let calculate single frame calibration estimator and a 90% confidence interval
+#' #Finally, let calculate SF calibration estimator and a 90% confidence interval
 #' #for population total for variable Feeding, considering Income and Metres2 as auxiliary 
 #' #variables and with frame sizes and overlap domain size known.
-#' CalSF(dataA$Feeding, dataB$Feeding, PiklA, PiklB, dataA$ProbB, dataB$ProbA, dataA$Domain, 
-#' dataB$Domain, N_A = 1735, N_B =  1191, N_ab = 601, xsAFrameA = dataA$Income, 
-#' xsBFrameA = dataB$Income, xsAFrameB = dataA$Metres2, xsBFrameB = dataB$Metres2, 
+#' CalSF(DatA$Feed, DatB$Feed, PiklA, PiklB, DatA$ProbB, DatB$ProbA, DatA$Domain, 
+#' DatB$Domain, N_A = 1735, N_B =  1191, N_ab = 601, xsAFrameA = DatA$Inc, 
+#' xsBFrameA = DatB$Inc, xsAFrameB = DatA$M2, xsBFrameB = DatB$M2, 
 #' XA = 4300260, XB = 176553, conf_level = 0.90)
 #' @export
 CalSF = function (ysA, ysB, pi_A, pi_B, pik_ab_B, pik_ba_A, domains_A, domains_B, N_A = NULL, N_B = NULL, N_ab = NULL, xsAFrameA = NULL, xsBFrameA = NULL, xsAFrameB = NULL, xsBFrameB = NULL, xsT = NULL, XA = NULL, XB = NULL, X = NULL, met = "linear", conf_level = NULL)
@@ -114,6 +117,10 @@ CalSF = function (ysA, ysB, pi_A, pi_B, pik_ab_B, pik_ba_A, domains_A, domains_B
 		stop("There are missing values in domains from frame A.")
 	if (any(is.na(domains_B)))
 		stop("There are missing values in domains from frame B.")
+	if (any(is.na(pik_ab_B[domains_A == "ab"])))
+		stop("Some values in pik_ab_B are 0 when they should not.")
+	if (any(is.na(pik_ba_A[domains_B == "ba"])))
+		stop("Some values in pik_ba_A are 0 when they should not.")
 	if (nrow(ysA) != nrow(pi_A) | nrow(ysA) != length(domains_A) | length(domains_A) != nrow(pi_A))
 		stop("Arguments from frame A have different sizes.")
 	if (nrow(ysB) != nrow(pi_B) | nrow(ysB) != length(domains_B) | length(domains_B) != nrow(pi_B))
@@ -122,42 +129,41 @@ CalSF = function (ysA, ysB, pi_A, pi_B, pik_ab_B, pik_ba_A, domains_A, domains_B
 		stop("Domains from frame A are not correct.")
 	if (length(which(domains_B == "b")) + length(which(domains_B == "ba")) != length(domains_B))
 		stop("Domains from frame B are not correct.")
-	if ((is.null (N_A) == "TRUE" & is.null (N_B) == "FALSE") | (is.null (N_A) == "FALSE" & is.null (N_B) == "TRUE"))
+	if ((is.null (N_A) & !is.null (N_B)) | (!is.null (N_A) & is.null (N_B)))
 		stop("Only one value has been indicated for N_A and N_B. This is not valid.") 
 	if ((is.null (xsAFrameA) & !is.null (xsBFrameA)) | (!is.null (xsAFrameA) & is.null (xsBFrameA)))
 		stop("Auxiliary information from frame A is available only in one frame. This is not a possible option.")
 	if ((is.null (xsAFrameB) & !is.null (xsBFrameB)) | (!is.null (xsAFrameB) & is.null (xsBFrameB)))
 		stop("Auxiliary information from frame B is available only in one frame. This is not a possible option.")
 
-	if (is.null(conf_level)) {
-		r <- 2
-		rnames <- c("Total", "Mean")
-	}
-	else { 
-		r <- 6
-		rnames <- c("Total", "Upper End", "Lower End", "Mean", "Upper End", "Lower End")
-	}
+	cl <- match.call()
 
 	sample <- rbind(ysA, ysB)
 	n_A <- nrow(ysA)
  	n_B <- nrow(ysB)
 	n <- n_A + n_B
 	c <- ncol(ysA)
-	results <- matrix(NA, nrow = r, ncol = c)
- 	rownames(results) <- rnames
-	colnames(results) <- cnames
 
 	domains <- factor(c(as.character(domains_A), as.character(domains_B)))  
 	ysA <- cbind(rep(1, n_A), ysA)
 	ysB <- cbind(rep(1, n_B), ysB)
 	sample <- rbind(ysA, ysB)
-	c1 <- ncol(sample)
 
 	delta_a <- Domains (rep (1, n), domains, "a")
 	delta_ab <- Domains (rep (1, n), domains, "ab")
 	delta_b <- Domains (rep (1, n), domains, "b")
 	delta_ba <- Domains (rep (1, n), domains, "ba")
 
+	est <- matrix(, 2, c, dimnames = list(c("Total", "Mean"), cnames))
+	varest <- matrix(, 2, c, dimnames = list(c("Var. Total", "Var. Mean"), cnames))
+	totdom <- NULL
+	meandom <- NULL
+	par <- 	NULL
+	if (is.null(conf_level))
+		interv <- NULL
+	else
+		interv <- matrix(, 6, c, dimnames = list(c("Total", "Lower Bound", "Upper Bound", "Mean", "Lower Bound", "Upper Bound"), cnames))
+
 	if (!is.null(dim(drop(pi_A))) & !is.null(dim(drop(pi_B)))) {
 
 		if (nrow(pi_A) != ncol(pi_A))
@@ -166,22 +172,11 @@ CalSF = function (ysA, ysB, pi_A, pi_B, pik_ab_B, pik_ba_A, domains_A, domains_B
 			stop("Pikl from frame B is not a square matrix.")
 		
 		pik_A <- diag(pi_A); pik_B <- diag(pi_B)
-		pik <- c(pik_A, pik_B)
-		w_tilde_iS_A <- rep(0, n_A)
-		for (i in 1:n_A)
-			if (domains_A[i] == "a")
-				w_tilde_iS_A[i] <- 1 / pik_A[i]
-			else
-				w_tilde_iS_A[i] <- 1 / (pik_A[i] + pik_ab_B[i])
-		w_tilde_iS_B <- rep(0, n_B)
-		for (i in 1:n_B)
-			if (domains_B[i] == "b")
-				w_tilde_iS_B[i] <- 1 / pik_B[i]
-			else
-				w_tilde_iS_B[i] <- 1 / (pik_B[i] + pik_ba_A[i])
+		w_tilde_iS_A <- (1 / diag(pi_A)) * (domains_A == "a") + (1 / (diag(pi_A) + pik_ab_B)) * (domains_A == "ab")
+		w_tilde_iS_B <- (1 / diag(pi_B)) * (domains_B == "b") + (1 / (diag(pi_B) + pik_ba_A)) * (domains_B == "ba")
 		d <- c(w_tilde_iS_A, w_tilde_iS_B)
 
-		for (k in 1:c1) {
+		for (k in 1:(c+1)) {
 
 			if (is.null(xsAFrameA) & is.null(xsBFrameB) & is.null(xsT)) {
 
@@ -189,14 +184,7 @@ CalSF = function (ysA, ysB, pi_A, pi_B, pik_ab_B, pik_ba_A, domains_A, domains_B
 				
 					if (is.null(N_A) & is.null(N_B)) {
 
-						Nhat_abS <- 0
-						for (i in 1:n_A)
-							if (domains_A[i] == "ab")
-								Nhat_abS <- Nhat_abS + 1 / (1 / pik_A[i] + 1 / pik_ab_B[i])
-				
-						for (i in 1:n_B)
-							if (domains_B[i] == "ba")
-								Nhat_abS <- Nhat_abS + 1 / (1 / pik_B[i] + 1 / pik_ba_A[i])
+						Nhat_abS <- sum((1 / (pik_A + pik_ab_B)) * (domains_A == "ab")) + sum((1 / (pik_B + pik_ba_A)) * (domains_B == "ba"))
 
 						Nhat_A <- HT (rep(1, n_A), pik_A)
 						Nhat_B <- HT (rep(1, n_B), pik_B)					
@@ -430,15 +418,20 @@ CalSF = function (ysA, ysB, pi_A, pi_B, pik_ab_B, pik_ba_A, domains_A, domains_B
 				total_estimation <- sum (g * d * sample[,k])
 
 			if (k > 1) {
+
 				mean_estimation <- total_estimation / size_estimation
-				results[,k-1] <- c(total_estimation, mean_estimation)
+				est[,k-1] <- c(total_estimation, mean_estimation)
+				Vhat_Yhat_CalSF <- varest(sample[,k], Xs, 1/d, g)
+				Vhat_Ymeanhat_CalSF <- 1/size_estimation^2 * Vhat_Yhat_CalSF
+				varest[,k-1] <- c(Vhat_Yhat_CalSF, Vhat_Ymeanhat_CalSF)
+
 				if (!is.null(conf_level)) {
-					Vhat_Yhat_CalSF <- varest(sample[,k], Xs, 1/d, g)
+
 					total_upper <- total_estimation + qnorm(1 - (1 - conf_level) / 2) * sqrt(Vhat_Yhat_CalSF)
 					total_lower <- total_estimation - qnorm(1 - (1 - conf_level) / 2) * sqrt(Vhat_Yhat_CalSF)
-					mean_upper <- mean_estimation + qnorm(1 - (1 - conf_level) / 2) * sqrt(1/size_estimation^2 * Vhat_Yhat_CalSF)
-					mean_lower <- mean_estimation - qnorm(1 - (1 - conf_level) / 2) * sqrt(1/size_estimation^2 * Vhat_Yhat_CalSF)
-					results[,k-1] <- c(total_estimation, total_upper, total_lower, mean_estimation, mean_upper, mean_lower)	
+					mean_upper <- mean_estimation + qnorm(1 - (1 - conf_level) / 2) * sqrt(Vhat_Ymeanhat_CalSF)
+					mean_lower <- mean_estimation - qnorm(1 - (1 - conf_level) / 2) * sqrt(Vhat_Ymeanhat_CalSF)
+					interv[,k-1] <- c(total_estimation, total_lower, total_upper, mean_estimation, mean_lower, mean_upper)	
 				}
 			}
 		}
@@ -447,22 +440,11 @@ CalSF = function (ysA, ysB, pi_A, pi_B, pik_ab_B, pik_ba_A, domains_A, domains_B
 
 		if (is.null(dim(drop(pi_A))) & is.null(dim(drop(pi_B)))){
 
-			pik <- c(pi_A, pi_B)
-			w_tilde_iS_A <- rep(0, n_A)
-			for (i in 1:n_A)
-				if (domains_A[i] == "a")
-					w_tilde_iS_A[i] <- 1 / pi_A[i]
-				else
-					w_tilde_iS_A[i] <- 1 / (pi_A[i] + pik_ab_B[i])
-			w_tilde_iS_B <- rep(0, n_B)
-			for (i in 1:n_B)
-				if (domains_B[i] == "b")
-					w_tilde_iS_B[i] <- 1 / pi_B[i]
-				else
-					w_tilde_iS_B[i] <- 1 / (pi_B[i] + pik_ba_A[i])
+			w_tilde_iS_A <- (1 / pi_A) * (domains_A == "a") + (1 / (pi_A + pik_ab_B)) * (domains_A == "ab")
+			w_tilde_iS_B <- (1 / pi_B) * (domains_B == "b") + (1 / (pi_B + pik_ba_A)) * (domains_B == "ba")
 			d <- c(w_tilde_iS_A, w_tilde_iS_B)
 
-			for (k in 1:c1) {
+			for (k in 1:(c+1)) {
 
 				if (is.null(xsAFrameA) & is.null(xsBFrameB) & is.null(xsT)) {
 
@@ -470,14 +452,7 @@ CalSF = function (ysA, ysB, pi_A, pi_B, pik_ab_B, pik_ba_A, domains_A, domains_B
 				
 						if (is.null(N_A) & is.null(N_B)) {
 
-							Nhat_abS <- 0
-							for (i in 1:n_A)
-								if (domains_A[i] == "ab")
-									Nhat_abS <- Nhat_abS + 1 / (1 / pi_A[i] + 1 / pik_ab_B[i])
-				
-							for (i in 1:n_B)
-								if (domains_B[i] == "ba")
-									Nhat_abS <- Nhat_abS + 1 / (1 / pi_B[i] + 1 / pik_ba_A[i])
+							Nhat_abS <- sum((1 / (pi_A + pik_ab_B)) * (domains_A == "ab")) + sum((1 / (pi_B + pik_ba_A)) * (domains_B == "ba"))
 
 							Nhat_A <- HT (rep(1, n_A), pi_A)
 							Nhat_B <- HT (rep(1, n_B), pi_B)
@@ -714,15 +689,20 @@ CalSF = function (ysA, ysB, pi_A, pi_B, pik_ab_B, pik_ba_A, domains_A, domains_B
 					total_estimation <- sum (g * d * sample[,k])
 
 				if (k > 1) {
+
 					mean_estimation <- total_estimation / size_estimation
-					results[,k-1] <- c(total_estimation, mean_estimation)
+					est[,k-1] <- c(total_estimation, mean_estimation)
+					Vhat_Yhat_CalSF <- varest(sample[,k], Xs, 1/d, g)
+					Vhat_Ymeanhat_CalSF <- 1/size_estimation^2 * Vhat_Yhat_CalSF
+					varest[,k-1] <- c(Vhat_Yhat_CalSF, Vhat_Ymeanhat_CalSF)
+
 					if (!is.null(conf_level)) {
-						Vhat_Yhat_CalSF <- varest(sample[,k], Xs, 1/d, g)
+
 						total_upper <- total_estimation + qnorm(1 - (1 - conf_level) / 2) * sqrt(Vhat_Yhat_CalSF)
 						total_lower <- total_estimation - qnorm(1 - (1 - conf_level) / 2) * sqrt(Vhat_Yhat_CalSF)
-						mean_upper <- mean_estimation + qnorm(1 - (1 - conf_level) / 2) * sqrt(1/size_estimation^2 * Vhat_Yhat_CalSF)
-						mean_lower <- mean_estimation - qnorm(1 - (1 - conf_level) / 2) * sqrt(1/size_estimation^2 * Vhat_Yhat_CalSF)
-						results[,k-1] <- c(total_estimation, total_upper, total_lower, mean_estimation, mean_upper, mean_lower)	
+						mean_upper <- mean_estimation + qnorm(1 - (1 - conf_level) / 2) * sqrt(Vhat_Ymeanhat_CalSF)
+						mean_lower <- mean_estimation - qnorm(1 - (1 - conf_level) / 2) * sqrt(Vhat_Ymeanhat_CalSF)
+						interv[,k-1] <- c(total_estimation, total_lower, total_upper, mean_estimation, mean_lower, mean_upper)	
 					}
 				}
 			}
@@ -730,5 +710,8 @@ CalSF = function (ysA, ysB, pi_A, pi_B, pik_ab_B, pik_ba_A, domains_A, domains_B
 		else
 			stop("Invalid option: Probability vector in one frame and probability matrix in the other frame. Type of both structures must match.")
 	}
-	return (results)
+   	results = list(Call = cl, Est = est, VarEst = varest, TotDomEst = totdom, MeanDomEst = meandom, Param = par, ConfInt = interv)
+   	class(results) = "EstimatorDF"
+   	attr(results, "attributesDF") = conf_level
+   	return(results)
 }
\ No newline at end of file
